from typing import Optional

from bs4 import BeautifulSoup


def slice_ponencia(corpus: str, start_string: Optional[str]) -> Optional[str]:
    """
    1. The `writer` block or the `category` block can be the `start string`
    2. Slice corpus from `start string` to the first <hr> tag

    Being able to generate an accurate slice of the corpus ensures better segmentation, such as:
    1. Determining the ruling portion based on the hr tag
    2. Itemizing the different ponencia segments
    3. Getting the footnotes section for each segment
    """
    # check if elements existing
    if not corpus or not start_string:
        return None

    # get the start
    if not (start_mark := corpus.find(start_string)):
        return None

    # add match to length of match
    start = start_mark + len(start_string)

    # get the end, i.e. the first hr tag found
    html = BeautifulSoup(corpus, "html5lib")
    if not (find_end := html.find("hr")):
        return None

    # convert to string (note unique id from dividers)
    str_hr_tag = str(find_end)
    if not (end := corpus.index(str_hr_tag)):
        return None

    # return slice
    return corpus[start:end]
